<?php
// +-----------------------------------------------------------------+
// |                   PhreeBooks Open Source ERP                    |
// +-----------------------------------------------------------------+
// | Copyright (c) 2008, 2009, 2010 PhreeSoft, LLC                   |
// | http://www.PhreeSoft.com                                        |
// +-----------------------------------------------------------------+
// | This program is free software: you can redistribute it and/or   |
// | modify it under the terms of the GNU General Public License as  |
// | published by the Free Software Foundation, either version 3 of  |
// | the License, or any later version.                              |
// |                                                                 |
// | This program is distributed in the hope that it will be useful, |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of  |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the   |
// | GNU General Public License for more details.                    |
// |                                                                 |
// | The license that is bundled with this package is located in the |
// | file: /doc/manual/ch01-Introduction/license.html.               |
// | If not, see http://www.gnu.org/licenses/                        |
// +-----------------------------------------------------------------+
//  Path: /includes/application_top.php
//

// Start the clock for the page parse time log
define('PAGE_EXECUTION_START_TIME', microtime(true));
// set php_self in the local scope
if (!isset($PHP_SELF)) $PHP_SELF = $_SERVER['PHP_SELF'];
// Check for application configuration parameters
if (file_exists('includes/configure.php')) {
  require('includes/configure.php');
} elseif (file_exists('install/index.php')) {
  header('Location: install/index.php');
  exit();
} else die('Phreedom cannot find the configuration file. Aborting!');
// Load some path constants
$path = (ENABLE_SSL_ADMIN == 'true' ? HTTPS_SERVER : HTTP_SERVER) . DIR_WS_ADMIN;
if (!defined('PATH_TO_MY_FILES')) define('PATH_TO_MY_FILES','my_files/');
define('DIR_WS_FULL_PATH', $path);	// full http path (or https if secure)
define('DIR_WS_MODULES',   'modules/');
define('DIR_WS_MY_FILES',  PATH_TO_MY_FILES);
// load some file system constants
define('DIR_FS_INCLUDES',  DIR_FS_ADMIN . 'includes/');
define('DIR_FS_MODULES',   DIR_FS_ADMIN . 'modules/');
define('DIR_FS_MY_FILES',  DIR_FS_ADMIN . PATH_TO_MY_FILES);
define('DIR_FS_THEMES',    DIR_FS_ADMIN . 'themes/');
define('FILENAME_DEFAULT', 'index');
// set the type of request (secure or not)
$request_type = (strtolower($_SERVER['HTTPS']) == 'on' || $_SERVER['HTTPS'] == '1' || strstr(strtoupper($_SERVER['HTTP_X_FORWARDED_BY']),'SSL') || strstr(strtoupper($_SERVER['HTTP_X_FORWARDED_HOST']),'SSL')) ? 'SSL' : 'NONSSL';
// define how do we update currency exchange rates. Possible values are 'oanda' 'xe' or ''
define('CURRENCY_SERVER_PRIMARY', 'oanda');
define('CURRENCY_SERVER_BACKUP',  'xe');
// define the inventory types that are tracked in cost of goods sold
define('COG_ITEM_TYPES','si,sr,ms,mi,as,sa');
@ini_set('session.gc_maxlifetime', (SESSION_TIMEOUT_ADMIN < 900 ? (SESSION_TIMEOUT_ADMIN + 900) : SESSION_TIMEOUT_ADMIN));
session_start();
$session_started = true;
// set the language
if (isset($_GET['language'])) {
  $_SESSION['language'] = $_GET['language'];
} elseif (!$_SESSION['language']) {
  $_SESSION['language'] = defined('DEFAULT_LANGUAGE') ? DEFAULT_LANGUAGE : 'en_us';
}
// see if the user is logged in
$user_validated = ($_SESSION['admin_id']) ? true : false;
// load general language translation, Check for global define overrides first
$path = DIR_FS_MODULES . 'phreedom/custom/language/' . $_SESSION['language'] . '/language.php';
if (file_exists($path)) { include($path); }
$path = DIR_FS_MODULES . 'phreedom/language/' . $_SESSION['language'] . '/language.php';
if (file_exists($path)) { require_once($path); } 
  else { require_once(DIR_FS_MODULES . 'phreedom/language/en_us/language.php'); }
// define general functions and classes used application-wide
require_once(DIR_FS_MODULES  . 'phreedom/defaults.php');
require_once(DIR_FS_INCLUDES . 'common_functions.php');
require_once(DIR_FS_INCLUDES . 'common_classes.php');
// pull in the custom language over-rides for this module (to pre-define the standard language)
$custom_path = DIR_FS_MODULES . $module . '/custom/pages/' . $page . '/extra_defines.php';
if (file_exists($custom_path)) { include($custom_path); }
// determine what theme to use
if (isset($_POST['theme'])) {
  define('DIR_WS_THEMES', DIR_WS_ADMIN . 'themes/' . $_POST['theme'] . '/');
  $_SESSION['theme'] = $_POST['theme'];
} elseif (isset($_SESSION['theme'])) {
  define('DIR_WS_THEMES', DIR_WS_ADMIN . 'themes/' . $_SESSION['theme'] . '/');
} else {
  define('DIR_WS_THEMES', DIR_WS_ADMIN . 'themes/default/');
  $_SESSION['theme'] = 'default';
}
define('DIR_WS_IMAGES', DIR_WS_THEMES . 'images/');
if (file_exists(DIR_WS_THEMES . 'icons/')) {
  define('DIR_WS_ICONS',  DIR_WS_THEMES . 'icons/');
} else { // use default
  define('DIR_WS_ICONS',  DIR_WS_ADMIN . 'themes/default/icons/');
}

$messageStack = new messageStack;
$toolbar      = new toolbar;
// determine what company to connect to
$db_company = (isset($_SESSION['company'])) ? $_SESSION['company'] : $_SESSION['companies'][$_POST['company']];
if ($db_company && file_exists(DIR_FS_MY_FILES . $db_company . '/config.php')) {
  define('DB_DATABASE', $db_company);
  require(DIR_FS_MY_FILES . $db_company . '/config.php');
  define('DB_SERVER_HOST',DB_SERVER); // for old PhreeBooks installs
  // Load queryFactory db classes
  require(DIR_FS_INCLUDES . 'db/' . DB_TYPE . '/query_factory.php');
  $db = new queryFactory();
  $db->connect(DB_SERVER_HOST, DB_SERVER_USERNAME, DB_SERVER_PASSWORD, DB_DATABASE);
  // set application wide parameters for phreebooks module
  $result = $db->Execute_return_error("select configuration_key, configuration_value from " . DB_PREFIX . "configuration");
  if ($db->error_number) die(LOAD_CONFIG_ERROR);
  while (!$result->EOF) {
	define($result->fields['configuration_key'], $result->fields['configuration_value']);
	$result->MoveNext();
  }
  // search the list modules and load configuration files and language files
  gen_pull_language('phreedom', 'menu');
  require_once(DIR_FS_MODULES . 'phreedom/config.php');
  $messageStack->debug_header();
  $loaded_modules = array();
  $dirs = scandir(DIR_FS_MODULES);
  foreach ($dirs as $dir) { // first pull all module language files, loaded or not
    if ($dir == '.' || $dir == '..') continue;
	if (is_dir(DIR_FS_MODULES . $dir)) gen_pull_language($dir, 'menu'); 
  }
  foreach ($dirs as $dir) {
    if ($dir == '.' || $dir == '..') continue;
    if (defined('MODULE_' . strtoupper($dir) . '_STATUS')) { // module is loaded
	  if ($dir <> 'phreedom') $loaded_modules[] = $dir;
      require_once(DIR_FS_MODULES . $dir . '/config.php');
    }
  }
  // pull in the custom language over-rides for this module (to pre-define the standard language)
  $path = DIR_FS_MODULES . $module . '/custom/pages/' . $page . '/extra_menus.php';
  if (file_exists($path)) { include($path); }
  $currencies = new currencies();
}

// some error checking before we proceed
// re-direct if SSL request not matching actual request
$prefered_type = ENABLE_SSL_ADMIN == 'true' ? 'SSL' : 'NONSSL';
if ($request_type <> $prefered_type) gen_redirect(html_href_link(FILENAME_DEFAULT, '', 'SSL'));
// check for default currency defined
if ($user_validated && !defined('DEFAULT_CURRENCY')) $messageStack->add(ERROR_NO_DEFAULT_CURRENCY_DEFINED, 'error');

?>