<?php
// +-----------------------------------------------------------------+
// |                   PhreeBooks Open Source ERP                    |
// +-----------------------------------------------------------------+
// | Copyright (c) 2008, 2009, 2010 PhreeSoft, LLC                   |
// | http://www.PhreeSoft.com                                        |
// +-----------------------------------------------------------------+
// | This program is free software: you can redistribute it and/or   |
// | modify it under the terms of the GNU General Public License as  |
// | published by the Free Software Foundation, either version 3 of  |
// | the License, or any later version.                              |
// |                                                                 |
// | This program is distributed in the hope that it will be useful, |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of  |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the   |
// | GNU General Public License for more details.                    |
// |                                                                 |
// | The license that is bundled with this package is located in the |
// | file: /doc/manual/ch01-Introduction/license.html.               |
// | If not, see http://www.gnu.org/licenses/                        |
// +-----------------------------------------------------------------+
//  Path: /modules/general/classes/toolbar.php
//

// General functions used across modules. Divided into the following sections:
// Section 1. Class toolbar
// Section 2. Class splitPageResults
// Section 3. Class objectInfo
// Section 4. Class messageStack
// Section 5. Class ctlPanel
// Section 6. Class currencies
// Section 7. Class encryption

/**************************************************************************************************************/
// Section 1. Class toolbar
/**************************************************************************************************************/
class toolbar {
// class constructor
  function toolbar($id = '0') {
    // set up the default toolbar
	$this->id            = $id;
	$this->search_text   = '';
	$this->search_period = CURRENT_ACCOUNTING_PERIOD;
	$this->period_strict = true; // if set to true, the 'All' option is included
	$this->search_prefix = '';
    $this->icon_size     = 'large';	// default icon size (choice are small, medium, large)
  	$this->icon_list     = array();
	$this->icon_list['cancel'] = array(
	  'show'   => true, 
	  'icon'   => 'actions/edit-undo.png',
	  'params' => '', 
	  'text'   => TEXT_CANCEL, 
	  'order'  => 1,
	);
	$this->icon_list['open'] = array(
	  'show'   => true, 
	  'icon'   => 'actions/document-open.png',
	  'params' => '', 
	  'text'   => TEXT_OPEN, 
	  'order'  => 2,
	);
	$this->icon_list['save'] = array(
	  'show'   => true, 
	  'icon'   => 'devices/media-floppy.png',
	  'params' => '', 
	  'text'   => TEXT_SAVE, 
	  'order'  => 3,
	);
	$this->icon_list['delete'] = array(
	  'show'   => true, 
	  'icon'   => 'actions/edit-delete.png',
	  'params' => '', 
	  'text'   => TEXT_DELETE, 
	  'order'  => 4,
	);
	$this->icon_list['print'] = array(
	  'show'   => true, 
	  'icon'   => 'phreebooks/pdficon_large.gif',
	  'params' => '', 
	  'text'   => TEXT_PRINT, 
	  'order'  => 5,
	);
  }

  function add_icon($name, $params = '', $order = 98) { // adds some common icons, per request
	switch ($name) {
	  case 'back':
	  case 'previous':   $image = 'actions/go-previous.png';            $text = TEXT_BACK;       break;
	  case 'continue':
	  case 'next':       $image = 'actions/go-next.png';                $text = TEXT_CONTINUE;   break;
	  case 'copy':       $image = 'actions/edit-copy.png';              $text = TEXT_COPY;       break;
	  case 'edit':       $image = 'actions/edit-find-replace.png';      $text = TEXT_EDIT;       break;
	  case 'email':      $image = 'apps/internet-mail.png';             $text = GEN_EMAIL;       break;
	  case 'export':     $image = 'actions/format-indent-more.png';     $text = TEXT_EXPORT;     break;
	  case 'export_csv': $image = 'mimetypes/x-office-spreadsheet.png'; $text = TEXT_EXPORT_CSV; break;
	  case 'finish':     $image = 'actions/document-save.png';          $text = TEXT_FINISH;     break;
	  case 'import':     $image = 'actions/format-indent-less.png';     $text = TEXT_IMPORT;     break;
	  case 'new':        $image = 'actions/document-new.png';           $text = TEXT_NEW;        break;
	  case 'recur':      $image = 'actions/go-jump.png';                $text = TEXT_RECUR;      break;
	  case 'rename':     $image = 'apps/accessories-text-editor.png';   $text = TEXT_RENAME;     break;
	  case 'payment':    $image = 'apps/accessories-calculator.png';    $text = TEXT_PAYMENT;    break;
	  case 'ship_all':   $image = 'mimetypes/package-x-generic.png';    $text = TEXT_SHIP_ALL;   break;
	  case 'search':     $image = 'actions/system-search.png';          $text = TEXT_SEARCH;     break;
	  case 'update':     $image = 'apps/system-software-update.png';    $text = TEXT_UPDATE;     break;
	  default:           $image = 'emblems/emblem-important.png';       $text = $name . ' ICON NOT FOUND'; 
	}
	if ($image) {
	  $this->icon_list[$name] = array(
		'show'   => true, 
		'icon'   => $image,
		'params' => $params, 
		'text'   => $text, 
		'order'  => $order,
	  );
	}
  }

  function add_help($index = '', $order = 99) { // adds some common icons, per request
	$this->icon_list['help'] = array(
	  'show'   => true, 
	  'icon'   => 'apps/help-browser.png',
	  'params' => 'onclick="window.open(\'' . DIR_WS_FULL_PATH . 'modules/phreehelp/index.php?idx=' . $index . '\',\'help\',\'width=800,height=600,resizable=1,scrollbars=1,top=100,left=100\')"', 
	  'text'   => TEXT_HELP, 
	  'order'  => $order,
	);
  }

  function build_toolbar($add_search = false, $add_period = false, $cal_props = false) { // build the main toolbar
	global $messageStack;
    $output = '';
	if ($add_search) $output .= $this->add_search();
	if ($add_period) $output .= $this->add_period();
	if ($cal_props) $output  .= $this->add_date($cal_props);
	$output .= '<div id="tb_main_' . $this->id . '" class="toolbar">' . "\n";
	// Sort the icons by designated order
	$sort_arr = array();
    foreach($this->icon_list as $uniqid => $row) {
        foreach($row as $key => $value) {
            $sort_arr[$key][$uniqid] = $value;
        }
    }
	array_multisort($sort_arr['order'], SORT_ASC, $this->icon_list);
	foreach ($this->icon_list as $id => $icon) {
	  if ($icon['show']) {
	    $output .= html_icon($icon['icon'], $icon['text'], $this->icon_size, 'id ="tb_icon_' . $id . '" style="cursor:pointer;" ' . $icon['params']) . "\n";
	  }
	}
	$output .= '</div>' . "\n"; // end of the right justified icons
	// display alerts/error messages, if any
    if ($messageStack->size > 0) $output .= $messageStack->output();
	return $output;
  }

  function add_search() {
	$output = '<div id="tb_search_' . $this->id . '" class="toolbar_right">' . "\n";
	$output .= HEADING_TITLE_SEARCH_DETAIL . '<br />';
	$output .= html_input_field('search_text', $this->search_text, $params = '');
	if ($this->search_text) $output .= '&nbsp;' . html_icon('actions/view-refresh.png', TEXT_RESET, 'small', 'onclick="location.href = \'index.php?' . gen_get_all_get_params(array('search_text', 'search_period', 'search_date', 'list', 'action')) . '\';" style="cursor:pointer;"');
    $output .= '&nbsp;' . html_icon('actions/system-search.png', TEXT_SEARCH, 'small', 'onclick="searchPage(\'' . gen_get_all_get_params(array('search_text', 'list', 'action')) . '\')" style="cursor:pointer;"');
	$output .= '</div>' . "\n";
	return $output;
  }

  function add_period() {
	$output = '<div id="tb_period_' . $this->id . '" class="toolbar_right">' . "\n";
	$output .= TEXT_INFO_SEARCH_PERIOD_FILTER . '<br />' . "\n";
	$output .= html_pull_down_menu('search_period', gen_get_period_pull_down($this->period_strict), $this->search_period, 'onchange="periodPage(\'' . gen_get_all_get_params(array('action', 'list')) . '\')"');
	$output .= '</div>' . "\n";
	return $output;
  }

  function add_date($cal_props) {
	$output = '<div id="tb_date_' . $this->id . '" class="toolbar_right">' . "\n";
	$output .= TEXT_DATE . '<br />' . "\n";
	$output .= html_calendar_field($cal_props) . "\n";
	$output .= '</div>' . "\n";
	return $output;
  }

}

/**************************************************************************************************************/
// Section 2. Class splitPageResults
/**************************************************************************************************************/
  class splitPageResults {
    function splitPageResults(&$current_page_number, $max_rows_per_page, &$sql_query, &$query_num_rows) {
      global $db;
      $this->page_prefix         = '';
	  $this->jump_page_displayed = false;
      if (empty($current_page_number)) $current_page_number = 1;
      $pos_to       = strlen($sql_query);
      $pos_from     = strpos($sql_query, ' from', 0);
      $pos_group_by = strpos($sql_query, 'group by', $pos_from);
      if (($pos_group_by < $pos_to) && ($pos_group_by != false)) $pos_to = $pos_group_by;
      $pos_having   = strpos($sql_query, ' having', $pos_from);
      if (($pos_having < $pos_to) && ($pos_having != false)) $pos_to = $pos_having;
      $pos_order_by = strpos($sql_query, 'order by', $pos_from);
      if (($pos_order_by < $pos_to) && ($pos_order_by != false)) $pos_to = $pos_order_by;
      $count          = $db->Execute($sql_query);
      $query_num_rows = $count->RecordCount();
      $num_pages      = ceil($query_num_rows / $max_rows_per_page);
      if ($current_page_number > $num_pages) $current_page_number = $num_pages;
      $offset = ($max_rows_per_page * ($current_page_number - 1));
      if ($offset < 0) $offset = 0;
      $sql_query .= " limit " . $offset . ", " . $max_rows_per_page;
    }

    function display_links($query_numrows, $max_rows_per_page, $max_page_links, $current_page_number, $parameters = '', $page_name = 'list') {
	  // calculate number of pages needing links
      $num_pages = ceil($query_numrows / $max_rows_per_page);
	  if ($num_pages == 0) $num_pages++;
	  $this->num_pages = $num_pages; // save it for retrieval by the templates
      $pages_array = array();
      for ($i = 1; $i <= $num_pages; $i++) $pages_array[] = array('id' => $i, 'text' => $i);
      if ($num_pages > 1) {
        $display_links = '';
        if ($current_page_number > 1) {
		  $display_links .= html_icon('actions/media-skip-backward.png', TEXT_GO_FIRST, 'small', 'onclick="location.href = \'' . html_href_link(FILENAME_DEFAULT, gen_get_all_get_params(array('action')) . 'action=go_first', 'SSL') . '\'" style="cursor:pointer;"');
		  $display_links .= html_icon('phreebooks/media-playback-previous.png', TEXT_GO_PREVIOUS, 'small', 'onclick="location.href = \'' . html_href_link(FILENAME_DEFAULT, gen_get_all_get_params(array('action')) . 'action=go_previous', 'SSL') . '\'" style="cursor:pointer;"');
        } else {
		  $display_links .= html_icon('actions/media-skip-backward.png', '', 'small', '');
		  $display_links .= html_icon('phreebooks/media-playback-previous.png', '', 'small', '');
        }
        if (!$this->jump_page_displayed) { // only diplay pull down once (the rest are not read by browser)
		  $display_links .= sprintf(TEXT_RESULT_PAGE, html_pull_down_menu($page_name, $pages_array, $current_page_number, 'onchange="jumpToPage(\'' . gen_get_all_get_params(array('list', 'action')) . 'action=go_page\')"'), $num_pages);
		  $this->jump_page_displayed = true;
		} else {
		  $display_links .= sprintf(TEXT_RESULT_PAGE, $current_page_number, $num_pages);
		}
        if (($current_page_number < $num_pages) && ($num_pages != 1)) {
		  $display_links .= html_icon('actions/media-playback-start.png', TEXT_GO_NEXT, 'small', 'onclick="location.href = \'' . html_href_link(FILENAME_DEFAULT, gen_get_all_get_params(array('action')) . 'action=go_next', 'SSL') . '\'" style="cursor:pointer;"');
		  $display_links .= html_icon('actions/media-skip-forward.png', TEXT_GO_LAST, 'small', 'onclick="location.href = \'' . html_href_link(FILENAME_DEFAULT, gen_get_all_get_params(array('action')) . 'action=go_last', 'SSL') . '\'" style="cursor:pointer;"');
        } else {
		  $display_links .= html_icon('actions/media-playback-start.png', '', 'small', '');
		  $display_links .= html_icon('actions/media-skip-forward.png', '', 'small', '');
        }
      } else {
        $display_links = sprintf(TEXT_RESULT_PAGE, $num_pages, $num_pages);
      }
      return $display_links;
    }

    function display_ajax($query_numrows, $max_rows_per_page, $max_page_links, $current_page_number, $parameters = '', $page_name = 'list', $id = '') {
	  // calculate number of pages needing links
      $num_pages = ceil($query_numrows / $max_rows_per_page);
	  if ($num_pages == 0) $num_pages++;
	  $this->num_pages = $num_pages; // save it for retrieval by the templates
      $display_links   = '';
      $pages_array     = array();
      for ($i = 1; $i <= $num_pages; $i++) $pages_array[] = array('id' => $i, 'text' => $i);
      if ($num_pages > 1) {
        if ($current_page_number > 1) {
		  $display_links .= html_icon('actions/media-skip-backward.png', TEXT_GO_FIRST, 'small', 'onclick="tabPage(\'' . $id . '\', \'go_first\')" style="cursor:pointer;"');
		  $display_links .= html_icon('phreebooks/media-playback-previous.png', TEXT_GO_PREVIOUS, 'small', 'onclick="tabPage(\'' . $id . '\', \'go_previous\')" style="cursor:pointer;"');
        } else {
		  $display_links .= html_icon('actions/media-skip-backward.png', '', 'small', '');
		  $display_links .= html_icon('phreebooks/media-playback-previous.png', '', 'small', '');
        }
        if (!$this->jump_page_displayed) { // only diplay pull down once (the rest are not read by browser)
		  $display_links .= sprintf(TEXT_RESULT_PAGE, html_pull_down_menu($page_name, $pages_array, $current_page_number, 'onchange="tabPage(\'' . $id . '\', \'go_page\')"'), $num_pages);
		  $this->jump_page_displayed = true;
		} else {
		  $display_links .= sprintf(TEXT_RESULT_PAGE, $current_page_number, $num_pages);
		}
        if (($current_page_number < $num_pages) && ($num_pages != 1)) {
		  $display_links .= html_icon('actions/media-playback-start.png', TEXT_GO_NEXT, 'small', 'onclick="tabPage(\'' . $id . '\', \'go_next\')" style="cursor:pointer;"');
		  $display_links .= html_icon('actions/media-skip-forward.png', TEXT_GO_LAST, 'small', 'onclick="tabPage(\'' . $id . '\', \'go_last\')" style="cursor:pointer;"');
        } else {
		  $display_links .= html_icon('actions/media-playback-start.png', '', 'small', '');
		  $display_links .= html_icon('actions/media-skip-forward.png', '', 'small', '');
        }
      } else {
        $display_links .= sprintf(TEXT_RESULT_PAGE, $num_pages, $num_pages);
		$display_links .= html_hidden_field($page_name, '1');
      }
      return $display_links;
    }

    function display_count($query_numrows, $max_rows_per_page, $current_page_number, $text_output) {
      $to_num = ($max_rows_per_page * $current_page_number);
      if ($to_num > $query_numrows) $to_num = $query_numrows;
      $from_num = ($max_rows_per_page * ($current_page_number - 1));
      if ($to_num == 0) {
        $from_num = 0;
      } else {
        $from_num++;
      }
      return sprintf($text_output, $from_num, $to_num, $query_numrows);
    }
  }

/**************************************************************************************************************/
// Section 3. Class objectInfo
/**************************************************************************************************************/
class objectInfo {
  function objectInfo($object_array = array()) {
    if (is_array($object_array)) {
      reset($object_array);
      while (list($key, $value) = each($object_array)) $this->$key = db_prepare_input($value);
	}
  }
}

/**************************************************************************************************************/
// Section 4. Class messageStack
/**************************************************************************************************************/
  class messageStack {
    var $size = 0;
    function messageStack() {
      $this->errors     = array();
	  $this->debug_info = NULL;
	  if ($_SESSION['messageQueue']) {
		$this->errors = $_SESSION['messageQueue'];
		$this->size = sizeof($_SESSION['messageQueue']);
		unset($_SESSION['messageQueue']);
      }
      if ($_SESSION['messageToStack']) {
        for ($i = 0, $n = sizeof($_SESSION['messageToStack']); $i < $n; $i++) {
          $this->add($_SESSION['messageToStack'][$i]['text'], $_SESSION['messageToStack'][$i]['type']);
        }
        unset($_SESSION['messageToStack']);
      }
    }

    function add($message, $type = 'error') {
      if ($type == 'error') {
        $this->errors[] = array('params' => 'class="messageStackError"', 'text' => html_icon('emblems/emblem-unreadable.png', TEXT_ERROR) . '&nbsp;' . $message);
      } elseif ($type == 'warning') {
        $this->errors[] = array('params' => 'class="messageStackWarning"', 'text' => html_icon('emblems/emblem-important.png', TEXT_CAUTION) . '&nbsp;' . $message);
      } elseif ($type == 'success') {
	    if (!HIDE_SUCCESS_MESSAGES) $this->errors[] = array('params' => 'class="messageStackSuccess"', 'text' => html_icon('emotes/face-smile.png', TEXT_SUCCESS) . '&nbsp;' . $message);
      } elseif ($type == 'caution') {
        $this->errors[] = array('params' => 'class="messageStackCaution"', 'text' => html_icon('emblems/emblem-important.png', TEXT_CAUTION) . '&nbsp;' . $message);
      } else {
        $this->errors[] = array('params' => 'class="messageStackError"', 'text' => $message);
      }
      $this->size++;
	  return true;
    }

    function add_session($message, $type = 'error') {
      if (!$_SESSION['messageToStack']) $_SESSION['messageToStack'] = array();
      $_SESSION['messageToStack'][] = array('text' => $message, 'type' => $type);
    }

	function convert_add_to_session() {
	  $_SESSION['messageQueue'] = $this->errors;
	}

    function reset() {
      $this->errors = array();
      $this->size   = 0;
    }

    function output() {
	  $output = NULL;
      $this->table_data_parameters = 'class="messageBox"';
	  if (sizeof($this->errors) > 0) {
	    $output .= '<table border="0" width="100%" cellspacing="0" cellpadding="2" class="messageBox">' . chr(10);
		foreach ($this->errors as $value) {
		  $output .= '<tr><td ' . $value['params'] . '>' . $value['text'] . '</td></tr>' . chr(10);
		}
		$output .= '</table>' . chr(10);
	  }
	  unset($this->errors);
      return $output;
    }

	function debug_header() {
	  $this->debug_info .= "Trace information for debug purposes. Phreedom release " . MODULE_PHREEDOM_VERSION . ", generated " . date('Y-m-d H:i:s') . ".\n\n";
	  $this->debug_info .= "\nGET Vars = "  . arr2string($_GET);
	  $this->debug_info .= "\nPOST Vars = " . arr2string($_POST);
	}

	function debug($txt) {
	  $this->debug_info .= $txt;
	}

	function write_debug() {
	  if (strlen($this->debug_info) < 1) return;
      $filename = DIR_FS_MY_FILES . 'trace.txt';
      if (!$handle = fopen($filename, 'w')) return $this->add("Cannot open file ($filename)", "error");
      if (fwrite($handle, $this->debug_info) === false) return $this->add("Cannot write to file ($filename)","error");
      fclose($handle);
	  $this->debug_info = NULL;
	  $this->add("Successfully created trace.txt file.","success");
	}
  }

/**************************************************************************************************************/
// Section 5. Class ctlPanel
/**************************************************************************************************************/
class ctl_panel {
  function ctl_panel() {
  }

  function build_div($title, $contents, $controls) {
	$output = '<!--// start: ' . $this->dashboard_id . ' //-->' . chr(10);
	$output .= '<div id="' . $this->dashboard_id . '" class="modbox" style="position:relative;">' . chr(10);
	$output .= '<table width="100%" class="mhdr" cellspacing="0" cellpadding="0">' . chr(10);
	$output .= '<tr>' . chr(10);
	// heading text
	$output .= '<td width="90%" class="mttl">' . $title . '&nbsp;</td>' . chr(10);
	// edit/cancel image (text)
	$output .= '<td class="medit">' . chr(10);
	$output .= '  <a href="javascript:void(0)" class="el" onclick ="return box_edit(\'' . $this->dashboard_id . '\');">';
	$output .= html_icon('categories/preferences-system.png', TEXT_PROPERTIES, $size = 'small', '', '16', '16', $this->dashboard_id . '_add');
	$output .= '  </a>' . chr(10);
	$output .= '  <a href="javascript:void(0)" class="csl" onclick ="return box_cancel(\'' . $this->dashboard_id . '\');">';
	$output .= html_icon('status/dialog-error.png', TEXT_CANCEL, $size = 'small', '', '16', '16', $this->dashboard_id . '_can');
	$output .= '  </a>' . chr(10);
	$output .= '</td>' . chr(10);
	// minimize/maximize image
	$output .= '<td class="mttlz">' . chr(10);
	$output .= '<a href="javascript:void(0)" class="box minbox" id="' . $this->dashboard_id . '_min" onclick="this.blur(); return min_box(\'' . $this->dashboard_id . '\')">' . chr(10);
	$output .= html_icon('actions/list-remove.png', TEXT_COLLAPSE, $size = 'small', '', '16', '16', $this->dashboard_id . '_exp');
	$output .= '</a></td>' . chr(10);
	// delete image
	$output .= '<td class="mttld">' . chr(10);
	$output .= '<a href="javascript:void(0)" class="box delbox" id="' . $this->dashboard_id . '_del" onclick="return del_box(\'' . $this->dashboard_id . '\')">';
	$output .= html_icon('emblems/emblem-unreadable.png', TEXT_REMOVE, $size = 'small');
	$output .= '</a>' . chr(10);
	$output .= '</td></tr></table>' . chr(10);
	// box properties section
	$output .= '<table class="mehdr" cellspacing="0" cellpadding="0">' . chr(10);
	$output .= '<tr class="es">' . chr(10);
	$output .= '<td class="meditbox">' . chr(10);
	$output .= html_form($this->dashboard_id . '_frm', FILENAME_DEFAULT, gen_get_all_get_params(array('action'))) . chr(10);
	$output .= $this->build_move_buttons($this->column_id, $this->row_id);
	$output .= $controls . chr(10);
	$output .= '<input type="hidden" name="dashboard_id" value="' . $this->dashboard_id . '" />' . chr(10);
	$output .= '<input type="hidden" name="column_id" value="' . $this->column_id . '" />' . chr(10);
	$output .= '<input type="hidden" name="row_id" value="' . $this->row_id . '" />' . chr(10);
	$output .= '<input type="hidden" name="todo" id="' . $this->dashboard_id . '_action" value="save" />' . chr(10);
	$output .= '</form></td></tr></table>' . chr(10);
	// box Contents
	$output .= '<div class="row" id="' . $this->dashboard_id . '_body" style="overflow:hidden;">' . chr(10);
	$output .= $contents;
	$output .= '</div>';
	// finish it up
	$output .= '</div>' . chr(10);
	$output .= '<!--// end: ' . $this->dashboard_id . ' //--><br />' . chr(10) . chr(10);
	return $output;
  }

  function build_move_buttons($column_id, $row_id) {
	$output = '<table cellspacing="0" cellpadding="0"><tr>' . chr(10);
	// move button - Left
	if ($column_id > 1) {
	  $output .= '<td class="mttlz">' . chr(10);
	  $output .= '<a href="javascript:void(0)" class="box minbox" onclick="return move_box(\'' . $this->dashboard_id . '\', \'move_left\')">';
	  $output .= html_icon('actions/go-previous.png', TEXT_MOVE_LEFT, $size = 'small');
	  $output .= '</a>' . chr(10);
	  $output .= '</td>' . chr(10);
	}
	// move button - Right
	if ($column_id < MAX_CP_COLUMNS) {
	  $output .= '<td class="mttlz">' . chr(10);
	  $output .= '<a href="javascript:void(0)" class="box minbox" onclick="return move_box(\'' . $this->dashboard_id . '\', \'move_right\')">';
	  $output .= html_icon('actions/go-next.png', TEXT_MOVE_RIGHT, $size = 'small');
	  $output .= '</a>' . chr(10);
	  $output .= '</td>' . chr(10);
	}
	// move button - Up
	if ($row_id > 1) {
	  $output .= '<td class="mttlz">' . chr(10);
	  $output .= '<a href="javascript:void(0)" class="box minbox" onclick="return move_box(\'' . $this->dashboard_id . '\', \'move_up\')">';
	  $output .= html_icon('actions/go-up.png', TEXT_MOVE_UP, $size = 'small');
	  $output .= '</a>' . chr(10);
	  $output .= '</td>' . chr(10);
	}
	// move button - Down
	if ($row_id < $this->get_next_row($column_id) - 1) {
	  $output .= '<td class="mttlz">' . chr(10);
	  $output .= '<a href="javascript:void(0)" class="box minbox" onclick="return move_box(\'' . $this->dashboard_id . '\', \'move_down\')">';
	  $output .= html_icon('actions/go-down.png', TEXT_MOVE_DOWN, $size = 'small');
	  $output .= '</a>' . chr(10);
	  $output .= '</td>' . chr(10);
	}
	$output .= '</tr></table>';
	return $output;
  }

  function get_next_row($column_id = 1) {
	global $db;
	$result = $db->Execute("select max(row_id) as max_row from " . TABLE_USERS_PROFILES . " 
	  where user_id = " . $_SESSION['admin_id'] . " and menu_id = '" . $this->menu_id . "' and column_id = " . $column_id);
	return ($result->fields['max_row'] + 1);
  }

}

/**************************************************************************************************************/
// Section 6. Class currencies
/**************************************************************************************************************/
class currencies {
  var $currencies;
  function currencies() {
    global $db, $messageStack;
    $this->currencies = array();
    $currencies = $db->Execute("select * from " . TABLE_CURRENCIES);
    while (!$currencies->EOF) {
	  $this->currencies[$currencies->fields['code']] = array(
	    'title'           => $currencies->fields['title'],
	    'symbol_left'     => $currencies->fields['symbol_left'],
	    'symbol_right'    => $currencies->fields['symbol_right'],
	    'decimal_point'   => $currencies->fields['decimal_point'],
	    'thousands_point' => $currencies->fields['thousands_point'],
	    'decimal_places'  => $currencies->fields['decimal_places'],
	    'decimal_precise' => $currencies->fields['decimal_precise'],
	    'value'           => $currencies->fields['value'],
	  );
      $currencies->MoveNext();
    }
	if (DEFAULT_CURRENCY == '') { // do not put this in the translation file, it is loaded before the language file is loaded.
	  $messageStack->add('You do not have a default currency set, PhreeBooks requires a default currency to operate properly! Please set the default currency in Setup -> Currencies.', 'error');
	}
  }

  // omits the symbol_left and symbol_right (just the formattted number))
  function format($number, $calculate_currency_value = true, $currency_type = DEFAULT_CURRENCY, $currency_value = '') {
    if ($calculate_currency_value) {
      $rate = ($currency_value) ? $currency_value : $this->currencies[$currency_type]['value'];
      $format_string = number_format($number * $rate, $this->currencies[$currency_type]['decimal_places'], $this->currencies[$currency_type]['decimal_point'], $this->currencies[$currency_type]['thousands_point']);
    } else {
      $format_string = number_format($number, $this->currencies[$currency_type]['decimal_places'], $this->currencies[$currency_type]['decimal_point'], $this->currencies[$currency_type]['thousands_point']);
    }
    return $format_string;
  }

  // omits the symbol_left and symbol_right (just the formattted number to the precision number of decimals))
  function precise($number, $calculate_currency_value = true, $currency_type = DEFAULT_CURRENCY, $currency_value = '') {
    if ($calculate_currency_value) {
	  $rate = ($currency_value) ? $currency_value : $this->currencies[$currency_type]['value'];
	  $format_string = number_format($number * $rate, $this->currencies[$currency_type]['decimal_precise'], $this->currencies[$currency_type]['decimal_point'], $this->currencies[$currency_type]['thousands_point']);
    } else {
	  $format_string = number_format($number, $this->currencies[$currency_type]['decimal_precise'], $this->currencies[$currency_type]['decimal_point'], $this->currencies[$currency_type]['thousands_point']);
    }
    return $format_string;
  }

  function format_full($number, $calculate_currency_value = true, $currency_type = DEFAULT_CURRENCY, $currency_value = '', $output_format = PDF_APP) {
    if ($calculate_currency_value) {
	  $rate = ($currency_value) ? $currency_value : $this->currencies[$currency_type]['value'];
	  $format_number = number_format($number * $rate, $this->currencies[$currency_type]['decimal_places'], $this->currencies[$currency_type]['decimal_point'], $this->currencies[$currency_type]['thousands_point']);
    } else {
	  $format_number = number_format($number, $this->currencies[$currency_type]['decimal_places'], $this->currencies[$currency_type]['decimal_point'], $this->currencies[$currency_type]['thousands_point']);
    }
	$zero = number_format(0, $this->currencies[$currency_type]['decimal_places']); // to handle -0.00
	if ($format_number == '-'.$zero) $format_number = $zero;
	$format_string = $this->currencies[$currency_type]['symbol_left'] . ' ' . $format_number . ' ' . $this->currencies[$currency_type]['symbol_right'];
    switch ($output_format) {
	  case 'FPDF': // assumes default character set
	    $format_string = str_replace('&euro;', chr(128),  $format_string); // Euro
	    break;
	  default:
    }
    return $format_string;
  }

  function get_value($code) {
    return $this->currencies[$code]['value'];
  }

  function clean_value($number, $currency_type = DEFAULT_CURRENCY) {
    // converts the number to standard float format (period as decimal, no thousands separator)
    $temp  = str_replace($this->currencies[$currency_type]['thousands_point'], '', trim($number));
    $value = str_replace($this->currencies[$currency_type]['decimal_point'], '.', $temp);
    $value = preg_replace("/[^-0-9.]+/","",$value);
    return $value;
  }

  function build_js_currency_arrays() {
	$js_codes  = 'var js_currency_codes = new Array(';
	$js_values = 'var js_currency_values = new Array(';
	foreach ($this->currencies as $code => $values) {
		$js_codes  .= "'" . $code . "',";
		$js_values .= $this->currencies[$code]['value'] . ",";
	}
	$js_codes  = substr($js_codes, 0, -1) . ");";
	$js_values = substr($js_values, 0, -1) . ");";
	return $js_codes . chr(10) . $js_values . chr(10);
  }
}

/**************************************************************************************************************/
// Section 7. Class encryption
/**************************************************************************************************************/
class encryption {
  var $scramble1;
  var $scramble2;
  var $errors;
  var $adj;
  var $mod;

  function encryption () {
	$this->errors = array();
	$this->scramble1 = '! #$%&()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[]^_`abcdefghijklmnopqrstuvwxyz{|}~';
	$this->scramble2 = 'f^jAE]okIOzU[2&q1{3`h5w_794p@6s8?BgP>dFV=m D<TcS%Ze|r:lGK/uCy.Jx)HiQ!#$~(;Lt-R}Ma,NvW+Ynb*0X';
	if (strlen($this->scramble1) <> strlen($this->scramble2)) {
		trigger_error('** SCRAMBLE1 is not same length as SCRAMBLE2 **', E_USER_ERROR);
	}
	$this->adj = 1.75;
	$this->mod = 3;
  }

  function decrypt ($key, $source) {
	$this->errors = array();
	$fudgefactor = $this->_convertKey($key);
	if ($this->errors) return;
	if (empty($source)) {
	  $this->errors[] = 'No value has been supplied for decryption';
	  return;
	}
	$target  = null;
	$factor2 = 0;
	for ($i = 0; $i < strlen($source); $i++) {
	  $char2 = substr($source, $i, 1);
	  $num2 = strpos($this->scramble2, $char2);
	  if ($num2 === false) {
		$this->errors[] = "Source string contains an invalid character ($char2)";
		return;
	  }
	  $adj     = $this->_applyFudgeFactor($fudgefactor);
	  $factor1 = $factor2 + $adj;
	  $num1    = $num2 - round($factor1);
	  $num1    = $this->_checkRange($num1);
	  $factor2 = $factor1 + $num2;
	  $char1 = substr($this->scramble1, $num1, 1);
	  $target .= $char1;
//echo "char1=$char1, num1=$num1, adj= $adj, factor1= $factor1, num2=$num2, char2=$char2, factor2= $factor2<br />\n";
	}
	return rtrim($target);
  }

  function encrypt ($key, $source, $sourcelen = 0) {
	$this->errors = array();
	$fudgefactor  = $this->_convertKey($key);
	if ($this->errors) return;
	if (empty($source)) {
	  $this->errors[] = 'No value has been supplied for encryption';
	  return;
	}
	while (strlen($source) < $sourcelen) $source .= ' ';
	$target = null;
	$factor2 = 0;
	for ($i = 0; $i < strlen($source); $i++) {
	  $char1 = substr($source, $i, 1);
	  $num1 = strpos($this->scramble1, $char1);
	  if ($num1 === false) {
		$this->errors[] = "Source string contains an invalid character ($char1)";
		return;
	  }
	  $adj     = $this->_applyFudgeFactor($fudgefactor);
	  $factor1 = $factor2 + $adj;
	  $num2    = round($factor1) + $num1;
	  $num2    = $this->_checkRange($num2);
	  $factor2 = $factor1 + $num2;
	  $char2   = substr($this->scramble2, $num2, 1);
	  $target .= $char2;
//echo "char1=$char1, num1=$num1, adj= $adj, factor1= $factor1, num2=$num2, char2=$char2, factor2= $factor2<br />\n";
	}
	return $target;
  }

  function getAdjustment () {
	return $this->adj;
  }

  function getModulus () {
	return $this->mod;
  }

  function setAdjustment ($adj) {
    $this->adj = (float)$adj;
  }

  function setModulus ($mod) {
    $this->mod = (int)abs($mod);
  }

  function _applyFudgeFactor (&$fudgefactor) {
	$fudge = array_shift($fudgefactor);
	$fudge = $fudge + $this->adj;
	$fudgefactor[] = $fudge;
	if (!empty($this->mod)) if ($fudge % $this->mod == 0) $fudge = $fudge * -1;
	return $fudge;
  }

  function _checkRange ($num) {
	$num = round($num);
	$limit = strlen($this->scramble1);
	while ($num >= $limit) $num = $num - $limit;
	while ($num < 0) $num = $num + $limit;
	return $num;
  }

  function _convertKey ($key) {
	if (empty($key)) {
	  $this->errors[] = 'No value has been supplied for the encryption key';
	  return;
	}
	$array[] = strlen($key);
	$tot = 0;
	for ($i = 0; $i < strlen($key); $i++) {
	  $char = substr($key, $i, 1);
	  $num = strpos($this->scramble1, $char);
	  if ($num === false) {
		$this->errors[] = "Key contains an invalid character ($char)";
		return;
	  }
	  $array[] = $num;
	  $tot = $tot + $num;
	}
	$array[] = $tot;
	return $array;
  }
}

?>
