<?php
// +-----------------------------------------------------------------+
// |                   PhreeBooks Open Source ERP                    |
// +-----------------------------------------------------------------+
// | Copyright (c) 2007-2008 PhreeSoft, LLC                          |
// | http://www.PhreeSoft.com                                        |
// +-----------------------------------------------------------------+
// | This program is free software: you can redistribute it and/or   |
// | modify it under the terms of the GNU General Public License as  |
// | published by the Free Software Foundation, either version 3 of  |
// | the License, or any later version.                              |
// |                                                                 |
// | This program is distributed in the hope that it will be useful, |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of  |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the   |
// | GNU General Public License for more details.                    |
// |                                                                 |
// | The license that is bundled with this package is located in the |
// | file: /doc/manual/ch01-Introduction/license.html.               |
// | If not, see http://www.gnu.org/licenses/                        |
// +-----------------------------------------------------------------+
//  Path: /modules/phreebooks/pages/admin/pre_process.php
//

/**************   Check user security   *****************************/
$security_level = $_SESSION['admin_security'][SECURITY_ID_CONFIGURATION];
if ($security_level == 0) { // no permission to enter page, error and redirect to home page
  $messageStack->add_session(ERROR_NO_PERMISSION, 'error');
  gen_redirect(html_href_link(FILENAME_DEFAULT, '', 'SSL'));
}

/**************  include page specific files    *********************/
gen_pull_language($module);
gen_pull_language($module, 'admin');
require_once(DIR_FS_WORKING . 'functions/phreebooks.php');
require_once(DIR_FS_WORKING . 'classes/install.php');
require_once(DIR_FS_WORKING . 'classes/chart_of_accounts.php');
require_once(DIR_FS_WORKING . 'classes/tax_auths.php');
require_once(DIR_FS_WORKING . 'classes/tax_auths_vend.php');
require_once(DIR_FS_WORKING . 'classes/tax_rates.php');
require_once(DIR_FS_WORKING . 'classes/tax_rates_vend.php');

/**************   page specific initialization  *************************/
$error  = false; 
$action = isset($_GET['action']) ? $_GET['action'] : $_POST['todo'];

$install           = new phreebooks_admin();
$chart_of_accounts = new chart_of_accounts();
$tax_auths         = new tax_auths();
$tax_auths_vend    = new tax_auths_vend();
$tax_rates         = new tax_rates();
$tax_rates_vend    = new tax_rates_vend();

/***************   Act on the action request   *************************/
switch ($action) {
  case 'import':
	if ($security_level < 3) {
	  $messageStack->add_session(ERROR_NO_PERMISSION,'error');
	  gen_redirect(html_href_link(FILENAME_DEFAULT, gen_get_all_get_params(array('action')), 'SSL'));
	}
	$delete_chart = ($_POST['delete_chart']) ? true : false;
	$std_chart    = db_prepare_input($_POST['std_chart']);
	// first verify the file was uploaded ok
	if (!$std_chart) if (!validate_upload('file_name', 'text', 'txt')) break;
	if ($delete_chart) {
	  $result = $db->Execute("select id from " . TABLE_JOURNAL_MAIN . " limit 1");
	  if ($result->RecordCount() > 0) {
	    $messageStack->add(GL_JOURNAL_NOT_EMTPY,'error');
	    break;
	  }
	  $db->Execute("TRUNCATE TABLE " . TABLE_CHART_OF_ACCOUNTS);
	  $db->Execute("TRUNCATE TABLE " . TABLE_CHART_OF_ACCOUNTS_HISTORY);
	}
	$filename = $std_chart ? (DIR_FS_WORKING.'language/'.$_SESSION['language'].'/charts/'.$std_chart) : $_FILES['file_name']['tmp_name'];
	$accounts = xml_to_object(file_get_contents($filename));
	if (is_array($accounts->account)) foreach ($accounts->account as $account) {
	  $result = $db->Execute("select id from " . TABLE_CHART_OF_ACCOUNTS . " where id = '" . $account->id . "'");
	  if ($result->RecordCount() > 0) {
	    $messageStack->add(sprintf(GL_ACCOUNT_DUPLICATE, $account->id),'error');
		continue;
	  }
	  $sql_data_array = array(
	    'id'              => $account->id,
		'description'     => $account->description,
		'heading_only'    => $account->heading,
		'primary_acct_id' => $account->primary,
		'account_type'    => $account->type,
	  );
	  db_perform(TABLE_CHART_OF_ACCOUNTS, $sql_data_array, 'insert');
	}
	build_and_check_account_history_records();
	break;
  case 'save':
	if ($security_level < 3) {
	  $messageStack->add_session(ERROR_NO_PERMISSION,'error');
	  gen_redirect(html_href_link(FILENAME_DEFAULT, gen_get_all_get_params(array('action')), 'SSL')); 
	}
	// save general tab
	foreach ($install->keys as $key => $default) {
	  $field = strtolower($key);
      if (isset($_POST[$field])) write_configure($key, $_POST[$field]);
    }
	gen_redirect(html_href_link(FILENAME_DEFAULT, gen_get_all_get_params(array('action')), 'SSL'));
	$messageStack->add(PHREEBOOKS_CONFIG_SAVED,'success');
    break;
  case 'delete':
	if ($security_level < 4) {
	  $messageStack->add(ERROR_NO_PERMISSION,'error');
	  break;
	}
    $subject = $_POST['subject'];
    $id      = $_POST['rowSeq'];
	if (!$subject || !$id) break;
    if ($$subject->btn_delete($id)) $close_popup = true;
	break;
  default:
}

/*****************   prepare to display templates  *************************/
// build some general pull down arrays
$sel_yes_no = array(
 array('id' => '0', 'text' => TEXT_NO),
 array('id' => '1', 'text' => TEXT_YES),
);

$sel_gl_desc = array(
 array('id' => '0', 'text' => TEXT_NUMBER),
 array('id' => '1', 'text' => TEXT_DESCRIPTION),
 array('id' => '2', 'text' => TEXT_BOTH),
);

$sel_order_lines = array(
  array('id' => '0', 'text' => TEXT_DOUBLE_MODE),
  array('id' => '1', 'text' => TEXT_SINGLE_MODE),
);

$sel_inv_due = array( // invoice date versus due date for aging
 array('id' => '0', 'text' => BNK_INVOICE_DATE),
 array('id' => '1', 'text' => BNK_DUE_DATE),
);

// load available charts based on language
$charts = scandir(DIR_FS_WORKING . 'language/' . $_SESSION['language'] . '/charts');
$sel_chart = array(array('id' => '0', 'text' => TEXT_SELECT));
foreach ($charts as $chart) {
  if (strpos($chart, 'xml')) {
	$temp = xml_to_object(file_get_contents(DIR_FS_WORKING . 'language/' . $_SESSION['language'] . '/charts/' . $chart));
    $sel_chart[] = array('id' => $chart, 'text' => $temp->description);
  }
}

// some pre-defined gl accounts
$cash_chart = gen_coa_pull_down(2, false, true, false, $restrict_types = array(0));    // cash types only
$ar_chart   = gen_coa_pull_down(2, false, true, false, $restrict_types = array(2));    // ar types only
$ap_chart   = gen_coa_pull_down(2, false, true, false, $restrict_types = array(20));   // ap types only
$ocl_chart  = gen_coa_pull_down(2, false, true, false, $restrict_types = array(22));   // other current liability types only
$inc_chart  = gen_coa_pull_down(2, false, true, false, $restrict_types = array(30));   // income types only
$inv_chart  = gen_coa_pull_down(2, false, true, false, $restrict_types = array(4,34)); // inv, expenses types only

$include_header   = true;
$include_footer   = true;
$include_tabs     = true;
$include_calendar = false;
$include_template = 'template_main.php';
define('PAGE_TITLE', BOX_PHREEBOOKS_MODULE_ADM);

?>