<?php
// +-----------------------------------------------------------------+
// |                   PhreeBooks Open Source ERP                    |
// +-----------------------------------------------------------------+
// | Copyright (c) 2008, 2009, 2010 PhreeSoft, LLC                   |
// | http://www.PhreeSoft.com                                        |
// +-----------------------------------------------------------------+
// | This program is free software: you can redistribute it and/or   |
// | modify it under the terms of the GNU General Public License as  |
// | published by the Free Software Foundation, either version 3 of  |
// | the License, or any later version.                              |
// |                                                                 |
// | This program is distributed in the hope that it will be useful, |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of  |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the   |
// | GNU General Public License for more details.                    |
// |                                                                 |
// | The license that is bundled with this package is located in the |
// | file: /doc/manual/ch01-Introduction/license.html.               |
// | If not, see http://www.gnu.org/licenses/                        |
// +-----------------------------------------------------------------+
//  Path: /modules/phreedom/functions/general.php
//

function load_company_dropdown($include_select = false) {
  $output = array();
  $i = 1;
  $contents = scandir(DIR_FS_MY_FILES);
  foreach ($contents as $file) {
	if ($file <> '.' && $file <> '..' && is_dir(DIR_FS_MY_FILES . $file)) {
	  if (file_exists(DIR_FS_MY_FILES . $file . '/config.txt')) convert_cfg($file);
	  if (file_exists(DIR_FS_MY_FILES . $file . '/config.php')) {
		require_once(DIR_FS_MY_FILES . $file . '/config.php');
		$output[$i] = array(
		  'text' => constant($file . '_TITLE'), 
		  'file' => $file,
		);
		$i++;
	  }
	}
  }
  if ($include_select) $output[0] = array('text' => TEXT_NONE, 'file' => 'none');
  $the_list = array();
  foreach ($output as $key => $value) {
    $_SESSION['companies'][$key] = $value['file'];
    $the_list[] = array('id' => $key, 'text' => $value['text']);
  }
  return $the_list;
}

function load_language_dropdown($language_directory = 'modules/phreedom/language/') {
  $output   = array();
  $contents = scandir($language_directory);
  foreach ($contents as $lang) {
	if ($lang <> '.' && $lang <> '..' && file_exists($language_directory . $lang . '/language.php')) {
	  if ($config_file = file($language_directory . $lang . '/language.php')) {
	    foreach ($config_file as $line) {
		  if (strstr($line,'\'LANGUAGE\'') !== false) {
		    $start_pos     = strpos($line, ',') + 2;
		    $end_pos       = strpos($line, ')') + 1;
		    $language_name = substr($line, $start_pos, $end_pos - $start_pos);
		    break;
		  }
	    }
	    $output[$lang] = array('id' => $lang, 'text' => $language_name);
	  }
	}
  }
  return $output;
}

function load_theme_dropdown() {
  $output   = array();
  $contents = scandir(DIR_FS_THEMES);
  foreach ($contents as $theme) {
	if ($theme <> '.' && $theme <> '..' && file_exists(DIR_FS_THEMES . $theme . '/config.php')) {
	  if ($config_file = file(DIR_FS_THEMES . $theme . '/config.php')) {
		foreach ($config_file as $line) {
		  if (strstr($line,'\'THEME_NAME\'') !== false) {
			$start_pos  = strpos($line, ',') + 2;
			$end_pos    = strrpos($line, '\'');
			$theme_name = trim(substr($line, $start_pos, $end_pos - $start_pos));
			break;
		  }
		}
		$output[$theme] = array('id' => $theme, 'text' => $theme_name);
	  }
	}
  }
  return $output;
}

/**************************** admin functions ***********************************************/
function admin_check_versions($module, $prerequisites = NULL) {
  global $messageStack;
  $error = false;
  if (is_array($prerequisites) && sizeof($prerequisites) > 0) {
	foreach ($prerequisites as $mod => $version) {
	  if (!$cur_rev = constant('MODULE_' . strtoupper($mod) . '_VERSION')) {
	    $messageStack->add(sprintf(ERROR_MODULE_NOT_INSTALLED, $module, $mod),'error');
		$error = true;
	  } elseif ($cur_rev < $version) {
	    $messageStack->add(sprintf(ERROR_MODULE_VERSION_TOO_LOW, $module, $mod, $version, $cur_rev),'error');
		$error = true;
	  }
	}
  }
  return $error;
}

function admin_install_dirs($dirlist, $path) {
  global $messageStack;
  $error = false;
  if (is_array($dirlist)) foreach ($dirlist as $dir) {
	if (!file_exists($path . $dir)) {
	  if (!@mkdir($path . $dir, 0755, true)) $error = $messageStack->add(sprintf(ERROR_CANNOT_CREATE_MODULE_DIR, $path . $dir), 'error');
    }
  }
  return $error;
}

function admin_remove_dirs($dirlist) {
  global $messageStack;
  $error = false;
  if (is_array($dirlist)) {
    $temp = array_reverse($dirlist);
	foreach($temp as $directory) {
	  if (!@rmdir($directory)) {
	    $error = $messageStack->add(sprintf(ERROR_CANNOT_REMOVE_MODULE_DIR, $directory), 'error');
	  }
	}
  }
  return $error;
}

function admin_install_tables($tables) {
  global $db;
  $error = false;
  foreach ($tables as $table => $create_table_sql) {
    if (!db_table_exists($table)) {
	  if (!$db->Execute($create_table_sql)) $error = true;
	}
  }
  return $error;
}

function admin_remove_tables($tables) {
  global $db;
  $error = false;
  if (is_array($tables)) foreach($tables as $table) {
	if (db_table_exists($table)) $db->Execute('DROP TABLE ' . $table);
  }
  return $error;
}

function admin_add_report_heading($doc_title, $doc_group) {
  global $db;
  $result = $db->Execute("select id from " . TABLE_PHREEFORM . " where doc_group = '" . $doc_group . "'");
  if ($result->RecordCount() < 1) {
    $db->Execute("INSERT INTO ".TABLE_PHREEFORM." (parent_id, doc_type, doc_title, doc_group, doc_ext, security, create_date) VALUES 
      (0, '0', '" . $doc_title . "', '".$doc_group."', '0', 'u:0;g:0', now())");
    $id = db_insert_id();
  } else {
    $id = $result->fields['id'];
  }
  return $id;
}

function admin_add_report_folder($parent_id, $doc_title, $doc_group, $doc_ext) {
  global $db;
  $error = false;
  $result = $db->Execute("select id from " . TABLE_PHREEFORM . " 
    where doc_group = '" . $doc_group . "'" . " and doc_ext = '" . $doc_ext . "'");
  if ($result->RecordCount() < 1) {
    $db->Execute("INSERT INTO ".TABLE_PHREEFORM." (parent_id, doc_type, doc_title, doc_group, doc_ext, security, create_date) VALUES 
      (".$parent_id.", '0', '" . $doc_title . "', '".$doc_group."', '".$doc_ext."', 'u:0;g:0', now())");
    $id = db_insert_id();
  } else {
    $id = $result->fields['id'];
  }
  return $error;
}

function admin_add_reports($module, $save_path = PF_DIR_MY_REPORTS) {
  $error = false;
  if (file_exists(DIR_FS_MODULES . $module . '/language/' . $_SESSION['language'] . '/reports/')) {
    $read_path = DIR_FS_MODULES . $module . '/language/' . $_SESSION['language'] . '/reports/';
  } elseif (file_exists(DIR_FS_MODULES . $module . '/language/en_us/reports/')) {
    $read_path = DIR_FS_MODULES . $module . '/language/en_us/reports/';
  } else {
    return $error; // nothing to import
  }
  $files = scandir($read_path);
  foreach ($files as $file) if (strtolower(substr($file, -4)) == '.xml') {
    if (!ImportReport('', $file, $read_path, $save_path)) $error = true;
  }
  return $error;
}

/************************ install functions ******************************/
function install_build_co_config_file($company, $key, $value) {
  global $messageStack;
  $filename = DIR_FS_ADMIN . 'my_files/' . $company . '/config.php';
  if (file_exists($filename)) { // update
    $lines = file($filename);
    $found_it = false;
    for ($x = 0; $x < count($lines); $x++) {
	  if (strpos(substr($lines[$x], 0, strpos($lines[$x], ',')), $key)) {
	    $lines[$x] = "define('" . $key . "','" . $value . "');" . "\n";
	    $found_it = true;
	    break;
	  }
    }
    if (!$found_it) $lines[] = "define('" . $key . "','" . $value . "');" . "\n";
  } else { // create the config file, because it doesn't exist
    $lines = array();
    $lines[] = '<?php' . "\n";
    $lines[] = '/* config.php */' . "\n";
    $lines[] = "define('" . $key . "','" . $value . "');" . "\n";
  }
  $line = implode('', $lines);
  if (!$handle = @fopen($filename, 'w')) {
    $messageStack->add(sprintf(MSG_ERROR_CANNOT_WRITE, $filename), 'error');
    return false;
  }
  fwrite($handle, $line);
  fclose($handle);
  return true;
}

/***************************** import/export functions ******************************/
function load_module_xml($module) {
  $error = false;
  $result = trim(file_get_contents(DIR_FS_MODULES . $module . '/' . $module . '.xml'));
  if (!$output = xml_to_object($result)) return false;
  // fix some special cases, multi elements with single entries convert to arrays
  if (is_object($output->Module->Table)) $output->Module->Table = array($output->Module->Table);
  return $output;
}

function build_sample_xml($structure, $db_table) {
  $output = '';
  $temp = $structure->Module->Table;
  foreach ($structure->Module->Table as $table) if ($table->Name == $db_table) {
    $output   .= '<' . $table->TagName . '>' . ' // ' . $table->Description . chr(10);
	foreach ($table->Field as $field) if ($field->CanImport) {
	  $req = ($field->Required) ? ('[' . TEXT_REQUIRED . '] '): '';
	  $output .= '  <' . $field->TagName . '>' . $field->Type . '</' . $field->TagName . '>' . ' // ' . $req . $field->Description . chr(10);
	}
	// check dependent tables and add xml sample
    if (is_object($table->LinkTable)) $table->LinkTable = array($table->LinkTable);
	if (isset($table->LinkTable)) foreach ($table->LinkTable as $subtable) {
      foreach ($temp as $working) if ($subtable->Name == $working->Name) {
	    $output   .= '  <' . $working->TagName . '>' . ' // ' . $working->Description . chr(10);
	    foreach ($working->Field as $field) if ($field->CanImport) {
	      $req = ($field->Required) ? ('[' . TEXT_REQUIRED . '] '): '';
	      $output .= '    <' . $field->TagName . '>' . $field->Type . '</' . $field->TagName . '>' . ' // ' . $req . $field->Description . chr(10);
	    }
        $output   .= '  </' . $working->TagName . '>' . chr(10);
	  }
	}
    $output   .= '</' . $table->TagName . '>' . chr(10);
  }
  return $output;
}

function build_sample_csv($structure, $db_table) {
  $output = '';
  $legend = TEXT_LEGEND . chr(10); // 'Legend:'
  $temp = $structure->Module->Table;
  foreach ($structure->Module->Table as $table) if ($table->Name == $db_table) {
	foreach ($table->Field as $field) {
	  if ($field->CanImport) {
	    $req = ($field->Required) ? ('[' . TEXT_REQUIRED . '] '): '';
	    $output .= $field->TagName . ', '; 
	    $legend .= '"' . $field->TagName . ': (' . $field->Type . ') - ' . $req . $field->Description . '"' . chr(10);
	  }
	}
	// check dependent tables and add csv sample based on the max number of entries specified
    if (is_object($table->LinkTable)) $table->LinkTable = array($table->LinkTable);
	if (isset($table->LinkTable)) foreach ($table->LinkTable as $subtable) {
      foreach ($temp as $working) if ($subtable->Name == $working->Name) {
	    for($i = 1; $i <= MAX_IMPORT_CSV_ITEMS; $i++) {
		  foreach ($working->Field as $field) {
	        if ($field->CanImport) {
			  $req = ($field->Required) ? ('[' . TEXT_REQUIRED . '] '): '';
	          $output .= $field->TagName . '_' . $i .', '; 
	          if ($i == 1) $legend .= '"' . $field->TagName . '_X: (' . $field->Type . ') - ' . $req . $field->Description . '"' . chr(10);
	        }
		  }
		}
	  }
	}
  }
  return $output . chr(10) . chr(10) . $legend;
}

function table_import_xml($structure, $db_table, $filename) {
//echo 'structure = '; print_r($structure); echo '<br>';
  global $db;
  $data = file_get_contents($_FILES[$filename]['tmp_name'], "r");
  $temp = $structure->Module->Table;
  foreach ($structure->Module->Table as $table) if ($table->Name == $db_table) {
	$tbl_active = $table;
	$tbl_tagname = $table->TagName;
	break;
  }
  if (!$result = xml_to_object($data)) return false;
  // fix some special cases, multi elements with single entries convert to arrays
  if (is_object($result->$tbl_tagname)) $result = array($result);
  foreach ($result->$tbl_tagname as $entry) {
    $sql_array = array();
	foreach ($tbl_active->Field as $field) {
	  $tag = $field->TagName;
	  if (isset($entry->$tag)) $sql_array[$field->Name] = $entry->$tag;
	}
//echo 'sql_array to write to table ' . DB_PREFIX . $db_table . ': '; print_r($sql_array); echo '<br>';
	db_perform(DB_PREFIX . $db_table, $sql_array, 'insert');
	// fetch the id for use with dependent tables
	$id = db_insert_id();
	// now look into dependent tables
    if (is_object($tbl_active->LinkTable)) $tbl_active->LinkTable = array($tbl_active->LinkTable);
	if (isset($tbl_active->LinkTable)) foreach ($tbl_active->LinkTable as $subtable) {
	  $sub_sql_array = array();
	  $sub_sql_array[$subtable->DependentField] = $id;
	  $sub_table_name = $subtable->Name;
      foreach ($temp as $working) if ($subtable->Name == $working->Name) {
	    $subtag = $working->TagName;
	    foreach ($working->Field as $field) {
	      $fieldtag = $field->TagName;
		  if (isset($entry->$subtag->$fieldtag)) $sub_sql_array[$field->Name] = $entry->$subtag->$fieldtag;
	    }
	  }
//echo 'sql_array to write to subtable ' . DB_PREFIX . $sub_table_name . ': '; print_r($sub_sql_array); echo '<br><br>';
	  db_perform(DB_PREFIX . $sub_table_name, $sub_sql_array, 'insert');
	}
  }
}

function table_import_csv($structure, $db_table, $filename) {
//echo 'structure = '; print_r($structure); echo '<br>';
  global $db, $messageStack;
  $data = file($_FILES[$filename]['tmp_name']);
  // read the header and build array
  if (sizeof($data) < 2) {
    $messageStack->add('The number of lines in the file is to small, a csv file must contain a header line and at least on input line!','error');
	return false;
  }
  $header = csv_explode(trim(array_shift($data)));
//echo 'header = '; print_r($header); echo '<br>';
  // build the map structure
  $temp = $structure->Module->Table;
  $map_array = array();
  foreach ($structure->Module->Table as $table) if ($table->Name == $db_table) {
	foreach ($table->Field as $field) {
	  $key = array_search($field->TagName, $header);
	  if ($key !== false) $map_array[$key] = array('cnt' => 0, 'table' => $table->Name, 'field' => $field->Name);
	}
	break;
  }
  // build dependent map tables
  $ref_mapping = array();
  if (is_object($table->LinkTable)) $table->LinkTable = array($table->LinkTable);
  if (isset($table->LinkTable)) foreach ($table->LinkTable as $subtable) {
    foreach ($structure->Module->Table as $working) if ($subtable->Name == $working->Name) {
	  $ref_mapping[$subtable->Name] = array(
		'pri_field' => $subtable->PrimaryField,
		'ref_field' => $subtable->DependentField,
	  );
	  for ($i = 1; $i <= MAX_IMPORT_CSV_ITEMS; $i++) {
		foreach ($working->Field as $field) {
		  $key = array_search($field->TagName . '_' . $i, $header);
		  if ($key !== false) $map_array[$key] = array(
		    'cnt'   => $i,
			'table' => $subtable->Name,
			'field' => $field->Name,
		  );
	    }
	  }
	}
  }
  foreach ($data as $line) {
    if (!$line  = trim($line)) continue; // blank line
	$line_array = $map_array;
	$sql_array  = array();
	$working    = csv_explode($line);
echo 'working: '; print_r($working); echo '<br>';
    for ($i = 0; $i < sizeof($working); $i++) $line_array[$i]['value'] = $working[$i];
	foreach ($line_array as $value) {
	  $sql_array[$value['table']][$value['cnt']][$value['field']] = $value['value'];
	}
	foreach ($sql_array as $table => $count) {
	  foreach ($count as $cnt => $table_array) {
//echo 'inserting data: '; print_r($table_array); echo '<br>';
	    if ($cnt == 0) { // main record, fetch id afterwards
	      db_perform(DB_PREFIX . $table, $table_array, 'insert');
		  $id = db_insert_id();
		} else { // dependent table 
		  $data_present = false;
		  foreach ($table_array as $value) if (gen_not_null($value)) $data_present = true;
		  if ($data_present) {
		    $table_array[$ref_mapping[$table]['ref_field']] = $id;
	        db_perform(DB_PREFIX . $table, $table_array, 'insert');
		  }
		}
	  }
	}
  }
}

function table_export_xml($structure, $db_table) {
  global $db;
  $output = '';
  $temp   = $structure->Module->Table;
  foreach ($structure->Module->Table as $table) if ($table->Name == $db_table) {
    $tag_map = array();
	foreach ($table->Field as $field) $tag_map[$field->Name] = $field->TagName;
	$result = $db->Execute("select * from " . DB_PREFIX . $db_table);
    if ($result->RecordCount() > 0) while (!$result->EOF) {
	  $output   .= '<' . $table->TagName . '>' . chr(10);
	  foreach ($result->fields as $key => $value) {
	    $output .= '  <' . $tag_map[$key] . '>' . $value . '</' . $tag_map[$key] . '>' . chr(10);
	  }
      $output   .= '</' . $table->TagName . '>' . chr(10);
	  $result->MoveNext();
	}
  }
  return $output;
}

function table_export_csv($structure, $db_table) {
  global $db;
  $output = '';
  $header = false;
  $temp   = $structure->Module->Table;
  foreach ($structure->Module->Table as $table) if ($table->Name == $db_table) {
    $tag_map = array();
	foreach ($table->Field as $field) $tag_map[$field->Name] = $field->TagName;
	$result = $db->Execute("select * from " . DB_PREFIX . $db_table);
    if ($result->RecordCount() > 0) while (!$result->EOF) {
	  if (!$header) { // output the header
	    $temp    = array();
		foreach ($result->fields as $key => $value) $temp[] = $tag_map[$key];
	    $output .= implode(',', $temp) . chr(10);
	    $header  = true;
	  }
	  $temp = array();
	  foreach ($result->fields as $key => $value) {
	    $temp[] = (strpos($value, ',') !== false) ? ('"'.$value.'"') : $value;
	  }
	  $output .= implode(',', $temp) . chr(10);
	  $result->MoveNext();
	}
  }
  return $output;
}

function csv_explode($str, $delim = ',', $enclose = '"', $preserve = false){
	$results = preg_split("/,(?=(?:[^\"]*\"[^\"]*\")*(?![^\"]*\"))/", trim($str));
	return preg_replace("/^\"(.*)\"$/", "$1", $results);
} 

?>