<?php 
// +-----------------------------------------------------------------+
// |                   PhreeBooks Open Source ERP                    |
// +-----------------------------------------------------------------+
// | Copyright (c) 2008, 2009, 2010 PhreeSoft, LLC                   |
// | http://www.PhreeSoft.com                                        |
// +-----------------------------------------------------------------+
// | This program is free software: you can redistribute it and/or   |
// | modify it under the terms of the GNU General Public License as  |
// | published by the Free Software Foundation, either version 3 of  |
// | the License, or any later version.                              |
// |                                                                 |
// | This program is distributed in the hope that it will be useful, |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of  |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the   |
// | GNU General Public License for more details.                    |
// |                                                                 |
// | The license that is bundled with this package is located in the |
// | file: /doc/manual/ch01-Introduction/license.html.               |
// | If not, see http://www.gnu.org/licenses/                        |
// +-----------------------------------------------------------------+
//  Path: /modules/phreeform/custom/extra_phreeform.php
// 

gen_pull_language('contacts'); // required for calculating terms

$FormProcessing['rnd_dec']    = PB_PF_ROUND_DECIMAL;
$FormProcessing['rnd_pre']    = PB_PF_ROUND_PRECISE;
$FormProcessing['def_cur']    = PB_PF_DEF_CURRENCY;
$FormProcessing['null_dcur']  = PB_PF_NULL_DEF_CURRENCY;
$FormProcessing['posted_cur'] = PB_PF_POSTED_CURRENCY;
$FormProcessing['null_pcur']  = PB_PF_NULL_POSTED_CURRENCY;
$FormProcessing['terms']      = PB_PF_TERMS_TO_LANGUAGE;
$FormProcessing['ordr_qty']   = PB_PF_ORDER_QTY;
$FormProcessing['branch']     = PB_PF_CONTACT_ID;
$FormProcessing['rep_id']     = PB_PF_USER_NAME;
$FormProcessing['ship_name']  = PB_PF_SHIP_METHOD;
$FormProcessing['j_desc']     = PB_PF_JOURNAL_DESC;
$FormProcessing['coa_type']   = PB_PF_COA_TYPE_DESC;

// Extra form processing operations
function pf_extra_processing($strData, $Process) {
  global $currencies, $posted_currencies;
  switch ($Process) {
	case "rnd_dec":
	  if (!is_numeric($strData)) return $strData;
	    else return $currencies->format($strData);
	case "rnd_pre":
	  if (!is_numeric($strData)) return $strData;
	    else return $currencies->precise($strData);
	case "def_cur": 
	  if (!is_numeric($strData)) return $strData;
	    else return $currencies->format_full($strData, true, DEFAULT_CURRENCY, 1);
	case "null_dcur": 
	  if (!is_numeric($strData)) return $strData;
	    else return (!$strData) ? '' :$currencies->format_full($strData, true, DEFAULT_CURRENCY, 1);
	case "posted_cur": 
	  if (!is_numeric($strData)) return $strData;
	    else return $currencies->format_full($strData, true, $posted_currencies['currencies_code'], $posted_currencies['currencies_value']);
	case "null_pcur": 
	  if (!is_numeric($strData)) return $strData;
	    else return (!$strData) ? '' : $currencies->format_full($strData, true, $posted_currencies['currencies_code'], $posted_currencies['currencies_value']);
	case "terms":     return gen_terms_to_language($strData, $short = true, $type = 'ar');
	case "ordr_qty":  return pull_order_qty($strData);
	case "branch":    return pf_get_branch_name($strData);
	case "rep_id":    return pf_get_user_name($strData);
	case "ship_name": return pf_get_ship_name($strData);
	case "j_desc":    return defined('GEN_ADM_TOOLS_J' . str_pad($strdata, 2, '0', STR_PAD_LEFT)) ? constant('GEN_ADM_TOOLS_J' . str_pad($strdata, 2, '0', STR_PAD_LEFT)) : $strData;
	case "coa_type":  return pf_get_coa_type($strData);
	default: // Do nothing
  }
  return $strData; // No Process recognized, return original value
}

function pull_order_qty($ref_id = 0) {
  global $db, $ReportID;
  $sql = "select qty from " . TABLE_JOURNAL_ITEM  . " where id = " . (int)$ref_id;
  $result = $db->Execute($sql);
  if ($result->RecordCount() == 0) { // no sales/purchase order, was a direct invoice/POS
	$sql = "select qty from " . TABLE_JOURNAL_ITEM  . " where so_po_item_ref_id = " . (int)$ref_id . " and ref_id = " . $ReportID;
	$result = $db->Execute($sql);
  }
  return $result->fields['qty'];
}

function pf_get_branch_name($id) {
  global $db;
  if (!$id) return COMPANY_ID;
  $result = $db->Execute("select short_name from " . TABLE_CONTACTS . " where id = " . (int)$id);
  if ($result->RecordCount() == 0) return COMPANY_ID;
  return $result->fields['short_name'];
}

function pf_get_user_name($id) {
  global $db;
  if (!$id) return '';
  $result = $db->Execute("select display_name from " . TABLE_USERS . " where admin_id = " . (int)$id);
  if ($result->RecordCount() == 0) return '';
  return $result->fields['display_name'];
}

function pf_get_ship_name($id) {
  if (!$id) return '';
  $parts = explode(':', $id);
  if (!$parts[0]) return '';
  load_method_language(DIR_FS_MODULES . 'shipping/methods/', $parts[0]);
  return defined(constant($parts[0] . '_' . $parts[1])) ? constant($parts[0] . '_' . $parts[1]) : $id;
}

function pf_get_coa_type($id) {
  gen_pull_language('phreebooks', 'admin');
  include(DIR_FS_MODULES . 'phreebooks/defaults.php');
  return $coa_types_list[$id]['text'];
}

?>